import re

def stringContainsLetters(string: str):
    pattern = '[a-zA-Z]'
    answer = False

    if re.findall(pattern, string):
        answer = True

    return answer

def stringContainsNumbers(string: str):
    pattern = '[0-9]'
    answer = False

    if re.findall(pattern, string):
        answer = True

    return answer


def longestString(strings: list):
    previousString = ''
    longestString = ''
    for currentString in strings:
        if not previousString:
            longestString = currentString
            previousString = currentString
            continue

        if len(currentString) > len(longestString):
            longestString = currentString

        previousString = currentString

    return longestString








from termcolor import colored

def getVersion(versionString: str):
    return versionString.split(':')[1].strip()

def getNamespace(versionString: str):
    namespaceAndRepo = versionString.split(':')[0].split('/')
    namespace = ''

    if len(namespaceAndRepo) == 2:
        namespace = namespaceAndRepo[0]
    else:
        namespace = 'library'

    return namespace.strip()

def getRepository(versionString: str):
    namespaceAndRepo = versionString.split(':')[0].split('/')
    repo = ''

    if len(namespaceAndRepo) == 2:
        repo = namespaceAndRepo[1]
    else:
        repo = namespaceAndRepo[0]

    return repo.strip()

# ----------------------------------------------------------------------------

class DockerImage:
    versionString = ''
    filename = ''
    channel = ''
    group = ''

    namespace = ''
    repository = ''
    version = ''

    def __init__(self, versionString: str, channel: str, group: str, filename: str):
        self.versionString = versionString.strip()
        self.filename = filename
        self.channel = channel
        self.group = group

        self.namespace = getNamespace(versionString)
        self.version = getVersion(versionString)
        self.repository = getRepository(versionString)

    def show(self):
        print('versionString: ' + self.versionString)
        print('channel: ' + self.channel)
        print('group: ' + self.group)
        print('filename: ' + self.filename)
        print('namespace: ' + self.namespace)
        print('repository: ' + self.repository)
        print('version: ' + self.version)
        print('--------------------------------------------------------------')

    def getVersionString(self, newVersion: str):
        versionString = ''

        if self.namespace == 'library':
            versionString = self.repository + ':' + newVersion
        else:
            versionString = self.namespace + '/' + self.repository + ':' + newVersion

        return versionString


    def update(self, nextVersion: str, dry_run: bool):
        print('--------------------------------------------------------------')

        if nextVersion == self.version:
            print(self.namespace + '/' + self.repository + ': ' + \
                colored(self.version, 'green'))

        else:
            print(self.namespace + '/' + self.repository + ': ' + \
                colored(self.version, 'red') + ' -> ' + \
                colored(nextVersion, 'green'))

            if not dry_run:
                file = open(self.filename, 'rt')
                data = file.read()
                data = data.replace(self.versionString, self.getVersionString(nextVersion))
                file.close()

                file = open(self.filename, 'wt')
                file.write(data)
                file.close()




import glob
import os
import re

from doup import DockerImage
from doup import Utility

class DockerVersionGrepper:
    path = ''
    groupToUpdate = None

    def __init__(self, path: str, groupToUpdate: str):
        self.path = path
        self.groupToUpdate = groupToUpdate

    def getDockerImagesInPath(self):
        files = glob.iglob(self.path + '/**/*', recursive=True)
        dockerImages = []
        for file in files:
            dockerImages.extend(self.getDockerImagesInFile(file))

        return dockerImages

    def getDockerImagesInFile(self, filepath: str):
        dockerImages = []

        try:
            with open(filepath, "r") as currentFile:
                previousLine = ''
                for line in currentFile:
                    dockerImage = self.getDockerImage(filepath, line, previousLine)
                    previousLine = line

                    if dockerImage:
                        dockerImages.append(dockerImage)
        except UnicodeDecodeError:
            pass
        except IsADirectoryError:
            pass

        return dockerImages


    def getDockerImage(self, filepath: str, line: str, previousLine: str):
        version = self.getVersionString(line)
        channel  = self.getDockerImageChannel(previousLine)
        group = self.getGroup(previousLine)
        isPartOfGroup = not self.groupToUpdate or (self.groupToUpdate and self.groupToUpdate == group)

        if version and channel and isPartOfGroup:
            return DockerImage.DockerImage(version, channel, group, filepath)

        return None

    def getVersionString(self, string: str):
        pattern = "\\s[\\w\\-]+[/\\w\\-]+:[\\w\\-\\.]+$"
        isMatch = re.search(pattern, string)
        version = ''

        if isMatch:
            matchGroup = isMatch.group()
            if self.isValidDockerImage(matchGroup):
                version = matchGroup

        return version


    def isValidDockerImage(self, matchGroup: str):
        containsLetters = Utility.stringContainsLetters(matchGroup)
        containsNumbers = Utility.stringContainsNumbers(matchGroup)

        if containsNumbers and containsLetters and len(matchGroup) < 60:
            return True

        return False

    def getDockerImageChannel(self, line: str):
        pattern = "doup:.*"
        isMatch = re.search(pattern, line)
        channel = ''

        if isMatch:
            channel = isMatch.group().split(':')[1]

        return channel


    def getGroup(self, line: str):
        pattern = "doup:.*"
        isMatch = re.search(pattern, line)
        group = ''

        if isMatch:
            groups = isMatch.group().split(':')
            if len(groups) == 3:
                group = groups[2]

        return group






import requests
import json
import re

from doup import DockerImage
from doup import Utility

# ----------------------------------------------------------------------------
# https://github.com/docker/hub-feedback/issues/1253
class DockerVersionUpdater:

    dry_run = False

    def __init__(self, dry_run: bool):
        self.dry_run = dry_run

    def updateDockerVersion(self, dockerImage: DockerImage.DockerImage):
        digest = self.getChannelDigest(dockerImage, 'amd64')
        tags = self.getTagsToDigest(dockerImage, digest)
        nextVersion = self.getLatestVersion(dockerImage, tags)

        dockerImage.update(nextVersion, self.dry_run)

    def getChannelDigest(self, dockerImage: DockerImage.DockerImage, architecture: str):
        url = "https://hub.docker.com/v2/repositories/" + \
            dockerImage.namespace + "/" + \
            dockerImage.repository + "/tags/" + \
            dockerImage.channel

        response = requests.get(url)
        bar = json.loads(response.text)
        images = bar['images']

        digest = ''
        for image in images:
            if image['architecture'] == architecture:
                digest = image['digest']

        return digest

    def getTagsToDigest(self, dockerImage: DockerImage.DockerImage, digest: str):
        url = "https://hub.docker.com/v2/repositories/" + \
            dockerImage.namespace + "/" + \
            dockerImage.repository + "/tags/?page_size=1000"

        response = json.loads(requests.get(url).text)
        tags = []
        for result in response['results']:
            for image in result['images']:
                try:
                    if image['digest'] == digest:
                        tags.append(result['name'])
                except:
                    version = dockerImage.repository + ':'  + result['name']
                    print('WARNING: image ' + version + ' has no key <digest>')
        return tags

    def getLatestVersion(self, dockerImage, tags: list):
        tagsToRemove = []
        for tag in tags:
            if not Utility.stringContainsNumbers(tag):
                tagsToRemove.append(tag)

        for tag in tagsToRemove:
            tags.remove(tag)

        latestVersion = Utility.longestString(tags)
        return latestVersion




import argparse
import os

from doup import DockerVersionGrepper
from doup import DockerVersionUpdater

def getArgs():
    currentPath = os.getcwd()
    parser = argparse.ArgumentParser(description='find docker images recursivly in the current working directory')
    parser.add_argument('-p', '--path', type=str,
                        default=currentPath,
                        help='search in a different path for docker versions')
    parser.add_argument('-d', '--dry-run', action='store_true',
                        help='run in dry run mode and dont change files')

    parser.add_argument('-g', '--group', type=str,
                        help='update only specific group members')

    args = parser.parse_args()

    return args

# -----------------------------------------------------------------------------

def main():
    args = getArgs()

    dockerVersionUpdater = DockerVersionUpdater.DockerVersionUpdater(args.dry_run)
    dockerVersionGrepper = DockerVersionGrepper.DockerVersionGrepper(args.path, args.group)

    dockerImages = dockerVersionGrepper.getDockerImagesInPath()
    for dockerImage in dockerImages:
        dockerVersionUpdater.updateDockerVersion(dockerImage)

